/* Copyright (C) 1995, Swedish Institute of Computer Science. */

/* Declarations & definitions for the SICStus Prolog C interface. */

/*----------------------------------------------------------------------*/
#ifndef _SICSTUS_H
#define _SICSTUS_H

#ifdef __cplusplus
extern "C" {
#endif

#define SPFliVersion 0x01010102

#if ___AIX__ || _AIX
#define PtrSPFuncs 1
#endif

#if __WATCOMC__ || __BORLANDC__ || _MSC_VER || __EMX__ || MACINTOSH || __IBMC__ || __IBMCPP__ || defined(__WIN32__)
#define DIFFCEXT 1
#define PtrSPFuncs 1
#endif

#if defined(__WATCOMC__) && defined(__NT__) || __BORLANDC__ || _MSC_VER 
# define SPCDECL __cdecl
#else
# if defined(__WATCOMC__) && defined(__OS2__)
#  define SPCDECL __syscall
# else
#  if (defined(__IBMC__) || defined(__IBMCPP__)) && defined(__OS2__)
#   define SPCDECL _System
#  else
#   define SPCDECL
#  endif
# endif
#endif

#ifndef SPEXP
#define SPEXP
#endif

#ifndef SPEXP1
#if defined(SPDLL) && (defined(__WATCOMC__) || defined(_MSC_VER))
#define SPEXP1 __declspec(dllexport)
#else
#define SPEXP1
#endif
#endif

#ifndef SPEXP2
#if defined(SPDLL) && defined(__BORLANDC__)
#define SPEXP2 __export
#else
#define SPEXP2
#endif
#endif

/*----------------------------------------------------------------------*/
/* Common */

#define SP_TYPE_VARIABLE 1
#define SP_TYPE_INTEGER 2
#define SP_TYPE_ATOM 3
#define SP_TYPE_FLOAT 4
#define SP_TYPE_COMPOUND 5

#define SP_SUCCESS 1
#define SP_FAILURE 0
#ifndef SP_ERROR
#define SP_ERROR (-1)
#endif

#define SP_WHEN_EXPLICIT 0
#define SP_WHEN_SAVE 1
#define SP_WHEN_RESTORE 1
#define SP_WHEN_EXIT 2

#if defined(__STDC__) || defined(__cplusplus) || __BORLANDC__ || _MSC_VER || __IBMC__
#define PROTOTYPE(argl) argl
#define ANYPOINTER void *
#define VA_PROTO1(Arg) (Arg, ...)
#define VA_PROTO2(Arg1, Arg2) (Arg1, Arg2, ...)
#define VA_PROTO3(Arg1, Arg2, Arg3) (Arg1, Arg2, Arg3, ...)
#else
#define PROTOTYPE(ignore) ()
#define ANYPOINTER char *
#define VA_PROTO1(Arg) ()
#define VA_PROTO2(Arg1, Arg2) ()
#define VA_PROTO3(Arg1, Arg2, Arg3) ()
#endif

#ifndef NULL
#define NULL 0
#endif

#define SP_initialize(Argc,Argv,BootPath) glue_initialize(Argc,Argv,BootPath,sp_pre_linkage,sp_pre_map)
#define SP_errno SP_get_errno()
#define SP_stdin SP_get_stdin()
#define SP_stdout SP_get_stdout()
#define SP_stderr SP_get_stderr()
#define SP_curin SP_get_curin()
#define SP_curout SP_get_curout()

#if HALF_TAGGED
typedef unsigned int SP_term;
#else
typedef unsigned long int SP_term;
#endif

/*----------------------------------------------------------------------*/
/* Types */

typedef long SP_qid;		/* Choice offset */
typedef char *SP_pred_ref;
typedef int SP_term_ref;
typedef int (SPCDECL SP_EventFun) PROTOTYPE((ANYPOINTER));
typedef int (SPCDECL SP_ReadHookProc) PROTOTYPE((int fd));
typedef void (SPCDECL SP_VoidFun) PROTOTYPE((void));
typedef void (SPCDECL SP_SigFun) PROTOTYPE((int));

/* Used by glue code */
typedef int (SPCDECL SP_GlueFun) PROTOTYPE(());
typedef void (SPCDECL SP_MainFun) PROTOTYPE(());
typedef void (SPCDECL SP_InitFun) PROTOTYPE((int));

/* Stream structure */

typedef struct stream_node SP_stream;

struct stream_node {
  struct stream_node *backward;
  struct stream_node *forward;

  char *filename;		/* empty string if no name */
  char *modename;		/* used by current_stream/3 */
  unsigned long mode;		/* bit vector */
/*  char *prompt; */
/*  int errno; */

  int fd;			/* file descriptor, -1 if none */
  int peek;			/* look ahead, -2 if none */
  struct stream_node *label;	/* Pointer to itself for error check */
  long prev_nl_pos;
  long last_nl_pos;
  long nl_count;
  long char_count;

  SP_GlueFun *sgetc;
  SP_GlueFun *sputc;
  SP_GlueFun *sflush;
  SP_GlueFun *seof;
  SP_MainFun *sclrerr;
  SP_GlueFun *sclose;
  unsigned long interrupt_flag;	/* 'off' or predicate name */
  ANYPOINTER user_handle;	/* pointer to additional stream data */
};

/*----------------------------------------------------------------------*/
/* Global vars defined in the "resource table" file used by Runtime
   Systems. */

extern SP_VoidFun *sp_pre_linkage[];
extern char *sp_pre_map[];

/*----------------------------------------------------------------------*/
/* External function declarations, spaux.h contains alternative
   definitions which are necessary for some platforms. Because of
   this, all function declarations, and nothing else, must be inside
   the following else-endif.
 */
#if defined(PtrSPFuncs) && defined(SPDLL)
#include <sicstus/spaux.h>
#else

#ifdef _MSC_VER
/* Defines funcs to have leading underscore (undecorated) to be
   compatible with Watcom and Borland. */
#ifdef _DATADEFS_H
#include "spmsc.h"
#else
#include <sicstus/spmsc.h>
#endif
#endif

/*----------------------------------------------------------------------*/
/* Preparation */

extern SPEXP int SPCDECL glue_initialize PROTOTYPE((int, char **, char *, SP_MainFun **, char **));
extern SPEXP void SPCDECL SP_force_interactive PROTOTYPE((void));
extern SPEXP int SPCDECL SP_load PROTOTYPE((char *));
extern SPEXP SP_pred_ref SPCDECL SP_predicate PROTOTYPE((char *,long,char *));
extern SPEXP SP_pred_ref SPCDECL SP_pred PROTOTYPE((unsigned long,long,unsigned long));
extern SPEXP SP_term_ref SPCDECL SP_new_term_ref PROTOTYPE((void));
extern SPEXP SP_term_ref SPCDECL SP_new_term_refs PROTOTYPE((int));
extern SPEXP void SPCDECL SP_reset_term_refs PROTOTYPE((SP_term_ref));


/*----------------------------------------------------------------------*/
/* Control */

extern SPEXP int SPCDECL SP_query PROTOTYPE(VA_PROTO1(SP_pred_ref predicate));
extern SPEXP int SPCDECL SP_query_cut_fail PROTOTYPE(VA_PROTO1(SP_pred_ref predicate));
extern SPEXP SP_qid SPCDECL SP_open_query PROTOTYPE(VA_PROTO1(SP_pred_ref predicate));
extern SPEXP SP_qid SPCDECL SP_open_query_array PROTOTYPE((SP_pred_ref, SP_term_ref *));
extern SPEXP int SPCDECL SP_next_solution PROTOTYPE((SP_qid));
extern SPEXP int SPCDECL SP_cut_query PROTOTYPE((SP_qid));
extern SPEXP int SPCDECL SP_close_query PROTOTYPE((SP_qid));
extern SPEXP int SPCDECL SP_event PROTOTYPE((SP_EventFun *, ANYPOINTER));
extern SPEXP void SPCDECL SP_continue PROTOTYPE((void));
extern SPEXP void SPCDECL SP_restart PROTOTYPE((void));
extern SPEXP void SPCDECL SP_ctrlc_action PROTOTYPE((void));


/*----------------------------------------------------------------------*/
/* SP_put_* */

extern SPEXP int SPCDECL SP_put_variable PROTOTYPE((SP_term_ref));
extern SPEXP int SPCDECL SP_put_term PROTOTYPE((SP_term_ref, SP_term_ref));
extern SPEXP int SPCDECL SP_put_integer PROTOTYPE((SP_term_ref, long));
extern SPEXP int SPCDECL SP_put_float PROTOTYPE((SP_term_ref, double));
extern SPEXP int SPCDECL SP_put_atom PROTOTYPE((SP_term_ref, unsigned long));
extern SPEXP int SPCDECL SP_put_string PROTOTYPE((SP_term_ref, char *));
extern SPEXP int SPCDECL SP_put_address PROTOTYPE((SP_term_ref, ANYPOINTER));
extern SPEXP int SPCDECL SP_put_list_chars PROTOTYPE((SP_term_ref, SP_term_ref, char *));
extern SPEXP int SPCDECL SP_put_number_chars PROTOTYPE((SP_term_ref, char *));
extern SPEXP int SPCDECL SP_put_functor PROTOTYPE((SP_term_ref, unsigned long, int));
extern SPEXP int SPCDECL SP_put_list PROTOTYPE((SP_term_ref));
extern SPEXP int SPCDECL SP_cons_functor PROTOTYPE(VA_PROTO3(SP_term_ref term, unsigned long name, int arity));
extern SPEXP int SPCDECL SP_cons_list PROTOTYPE((SP_term_ref, SP_term_ref, SP_term_ref));


/*----------------------------------------------------------------------*/
/* SP_get_* */

extern SPEXP int SPCDECL SP_get_integer PROTOTYPE((SP_term_ref, long *));
extern SPEXP int SPCDECL SP_get_float PROTOTYPE((SP_term_ref, double *));
extern SPEXP int SPCDECL SP_get_atom PROTOTYPE((SP_term_ref, unsigned long *));
extern SPEXP int SPCDECL SP_get_string PROTOTYPE((SP_term_ref, char **));
extern SPEXP int SPCDECL SP_get_address PROTOTYPE((SP_term_ref, ANYPOINTER *));
extern SPEXP int SPCDECL SP_get_list_chars PROTOTYPE((SP_term_ref, char **));
extern SPEXP int SPCDECL SP_get_number_chars PROTOTYPE((SP_term_ref, char **));
extern SPEXP int SPCDECL SP_get_functor PROTOTYPE((SP_term_ref, unsigned long *, int *));
extern SPEXP int SPCDECL SP_get_list PROTOTYPE((SP_term_ref, SP_term_ref, SP_term_ref));
extern SPEXP int SPCDECL SP_get_arg PROTOTYPE((int, SP_term_ref, SP_term_ref));


/*----------------------------------------------------------------------*/
/* SP_is_* */

extern SPEXP int SPCDECL SP_term_type PROTOTYPE((SP_term_ref));
extern SPEXP int SPCDECL SP_is_variable PROTOTYPE((SP_term_ref));
extern SPEXP int SPCDECL SP_is_integer PROTOTYPE((SP_term_ref));
extern SPEXP int SPCDECL SP_is_float PROTOTYPE((SP_term_ref));
extern SPEXP int SPCDECL SP_is_atom PROTOTYPE((SP_term_ref));
extern SPEXP int SPCDECL SP_is_compound PROTOTYPE((SP_term_ref));
extern SPEXP int SPCDECL SP_is_list PROTOTYPE((SP_term_ref));
extern SPEXP int SPCDECL SP_is_atomic PROTOTYPE((SP_term_ref));
extern SPEXP int SPCDECL SP_is_number PROTOTYPE((SP_term_ref));


/*----------------------------------------------------------------------*/
/* Others */

extern SPEXP unsigned long SPCDECL SP_atom_from_string PROTOTYPE((char *));
extern SPEXP char* SPCDECL SP_string_from_atom PROTOTYPE((unsigned long));
/* extern SPEXP int SPCDECL SP_atom_length PROTOTYPE((unsigned long)); */
extern SPEXP int SPCDECL SP_unify PROTOTYPE((SP_term_ref,SP_term_ref));
extern SPEXP int SPCDECL SP_compare PROTOTYPE((SP_term_ref,SP_term_ref));
extern SPEXP int SPCDECL SP_exception_term PROTOTYPE((SP_term_ref));
extern SPEXP void SPCDECL SP_raise_exception PROTOTYPE((SP_term_ref));
extern SPEXP char* SPCDECL SP_error_message PROTOTYPE((int));
extern SPEXP int SPCDECL SP_get_errno PROTOTYPE((void));


/*----------------------------------------------------------------------*/
/* Streams */

extern SPEXP SP_stream* SPCDECL SP_get_stdin PROTOTYPE((void));
extern SPEXP SP_stream* SPCDECL SP_get_stdout PROTOTYPE((void));
extern SPEXP SP_stream* SPCDECL SP_get_stderr PROTOTYPE((void));
extern SPEXP SP_stream* SPCDECL SP_get_curin PROTOTYPE((void));
extern SPEXP SP_stream* SPCDECL SP_get_curout PROTOTYPE((void));

extern SPEXP int SPCDECL SP_make_stream PROTOTYPE((
			  ANYPOINTER handle,
			  SP_GlueFun *sgetc,
			  SP_GlueFun *sputc,
			  SP_GlueFun *sflush,
			  SP_GlueFun *seof,
			  SP_MainFun *sclrerr,
			  SP_GlueFun *sclose,
			  SP_stream **stream_code));

extern SPEXP void SPCDECL SP_set_tty PROTOTYPE((SP_stream *));

extern SPEXP int SPCDECL SP_getc PROTOTYPE((void));
extern SPEXP int SPCDECL SP_fgetc PROTOTYPE((SP_stream *s));
extern SPEXP void SPCDECL SP_putc PROTOTYPE((int ch));
extern SPEXP void SPCDECL SP_fputc PROTOTYPE((int ch, SP_stream *s));
extern SPEXP void SPCDECL SP_puts PROTOTYPE((char *string));
extern SPEXP void SPCDECL SP_fputs PROTOTYPE((char *string, SP_stream *s));
extern SPEXP int SPCDECL SP_printf PROTOTYPE(VA_PROTO1(char *fmt));
extern SPEXP int SPCDECL SP_fprintf PROTOTYPE(VA_PROTO2(SP_stream *stream, char *fmt));
extern SPEXP int SPCDECL SP_fflush PROTOTYPE((SP_stream *s));
extern SPEXP int SPCDECL SP_fclose PROTOTYPE((SP_stream *s));


/*----------------------------------------------------------------------*/
/* Hooks */

/* signal() replacement */
extern SPEXP SP_SigFun* SPCDECL SP_signal PROTOTYPE((int, SP_SigFun *));

extern SPEXP SP_SigFun* SPCDECL SP_reinstall_signal PROTOTYPE((int, SP_SigFun *));

/* Called when reading TTY */
extern SPEXP SP_ReadHookProc * SPCDECL SP_set_read_hook
  PROTOTYPE((SP_ReadHookProc *));

extern SPEXP SP_VoidFun * SPCDECL SP_set_reinit_hook PROTOTYPE((SP_VoidFun *));

extern SPEXP SP_VoidFun * SPCDECL SP_set_interrupt_hook PROTOTYPE((SP_VoidFun *));

/*----------------------------------------------------------------------*/
/* Undocumented. The following definitions should be avoided if possible. */

extern SPEXP int SPCDECL SP_action PROTOTYPE((int, ANYPOINTER)); /* Perform some action */
extern SPEXP long SPCDECL SP_inquiry PROTOTYPE((int)); /* Get information from Prolog */
extern SPEXP int SPCDECL SP_choice_offset PROTOTYPE((int));
extern SPEXP void SPCDECL SP_put_choice_argument PROTOTYPE((int offset,int,SP_term_ref));
extern SPEXP void SPCDECL SP_write_term PROTOTYPE((SP_term_ref));


/*----------------------------------------------------------------------*/
/* Misc */

extern SPEXP int SPCDECL SP_isatty PROTOTYPE((SP_stream *));
extern SPEXP ANYPOINTER SPCDECL SP_malloc PROTOTYPE((unsigned int));
extern SPEXP ANYPOINTER SPCDECL SP_realloc PROTOTYPE((ANYPOINTER, unsigned int));
extern SPEXP void SPCDECL SP_free PROTOTYPE((ANYPOINTER));
extern SPEXP void SPCDECL SP_bcopy PROTOTYPE((ANYPOINTER src, ANYPOINTER dst, int));
extern SPEXP void SPCDECL SP_bzero PROTOTYPE((ANYPOINTER, int));

extern SPEXP void SPCDECL muse_init_lock PROTOTYPE((int *));
extern SPEXP void SPCDECL muse_lock PROTOTYPE((int *));
extern SPEXP void SPCDECL muse_un_lock PROTOTYPE((int *));
extern SPEXP int SPCDECL muse_max_workers PROTOTYPE((void));
extern SPEXP int SPCDECL muse_worker_id PROTOTYPE((void));
extern SPEXP int SPCDECL muse_num_workers PROTOTYPE((void));


/*----------------------------------------------------------------------*/
/* Raising standard errors from C-code */

/* SP_save_error() followed by SP_raise_error() will raise a standard
   error exception according to the constants defined below.
*/

extern SPEXP void SPCDECL SP_save_error PROTOTYPE((int error_type, char *error_msg, SP_term_ref culprit));
extern SPEXP void SPCDECL SP_raise_error PROTOTYPE((char *name, int arity, int argn));

extern SPEXP void SPCDECL sp_syserror PROTOTYPE((int api, char *msg, char *pred, char *syscall));


/*----------------------------------------------------------------------*/
/* Used by glue code */

extern SPEXP int SPCDECL glue_get_integer PROTOTYPE((SP_term, long *));
extern SPEXP int SPCDECL glue_get_float PROTOTYPE((SP_term, double *));
extern SPEXP int SPCDECL glue_get_atom PROTOTYPE((SP_term, unsigned long *));
extern SPEXP int SPCDECL glue_get_string PROTOTYPE((SP_term, char **));
extern SPEXP int SPCDECL glue_get_strn PROTOTYPE((int, SP_term, char *));
extern SPEXP int SPCDECL glue_get_address PROTOTYPE((SP_term, ANYPOINTER *));
extern SPEXP int SPCDECL glue_get_chars PROTOTYPE((SP_term, char **));

extern SPEXP int SPCDECL glue_make_integer PROTOTYPE((SP_term_ref, long));
extern SPEXP int SPCDECL glue_make_float PROTOTYPE((SP_term_ref, double));
extern SPEXP int SPCDECL glue_make_atom PROTOTYPE((SP_term_ref, unsigned long));
extern SPEXP int SPCDECL glue_make_str PROTOTYPE((SP_term_ref, char *));
extern SPEXP int SPCDECL glue_make_strn PROTOTYPE((int, SP_term_ref, char *));
extern SPEXP int SPCDECL glue_make_addr PROTOTYPE((SP_term_ref, char *));
extern SPEXP int SPCDECL glue_make_chars PROTOTYPE((SP_term_ref, char *));
extern SPEXP void SPCDECL glue_error PROTOTYPE((char *, int, int, int, SP_term));

extern SPEXP SP_term SPCDECL sp_ref_term PROTOTYPE((int ref));

/* New, added last to preserve old order */
extern SPEXP int SPCDECL SP_atom_length PROTOTYPE((unsigned long));
extern SPEXP int SPCDECL SP_put_list_n_chars PROTOTYPE((SP_term_ref, SP_term_ref, long, char *));
extern SPEXP int SPCDECL SP_get_list_n_chars PROTOTYPE((SP_term_ref, SP_term_ref, long, long *, char *));

/* End of external function declarations */
#endif /* PtrSPFuncs */

/*----------------------------------------------------------------------*/
/* Error codes used to raise_exception from C-code */
/* xref Bips/intrins1.pl */
#define INSTANTIATION_ERROR 0
#define TYPE_ERROR 1
#define DOMAIN_ERROR 2
#define REPRESENTATION_ERROR 3
#define EXISTENCE_ERROR 4
#define SYSTEM_ERROR 5
#define PERMISSION_ERROR 6


/* Some types, domains, etc. */
/* xref Bips/intrins1.pl */
#define TYPE_ATOMIC (1<<8)
#define TYPE_ATOM (2<<8)
#define TYPE_NUMBER (3<<8)
#define TYPE_INTEGER (4<<8)
#define TYPE_COMPOUND (5<<8)
#define TYPE_VAR (6<<8)
#define TYPE_CHARS (7<<8)

#define DOMAIN_1_ARITY (1<<8)
#define DOMAIN_CHAR (2<<8)
#define DOMAIN_NONZERO (3<<8)
#define DOMAIN_GT_ZERO (4<<8)
#define DOMAIN_GE_ZERO (5<<8)
#define DOMAIN_EXPRESSION (6<<8)
#define DOMAIN_LIST (7<<8)
#define DOMAIN_MUTABLE (8<<8)
#define DOMAIN_STREAM (9<<8)
#define DOMAIN_INPUT_STREAM (10<<8)
#define DOMAIN_OUTPUT_STREAM (11<<8)
#define DOMAIN_ATTR_MODULE (12<<8)
#define DOMAIN_IO_DESCR_STREAM (13<<8)
#define DOMAIN_ABS_LE_ONE (14<<8)
#define DOMAIN_ABS_GE_ONE (15<<8)
#define DOMAIN_GE_ONE (16<<8)
#define DOMAIN_GT_FZERO (17<<8)
#define DOMAIN_GE_FZERO (18<<8)
  
#define REPR_MAXARITY (1<<8)
#define REPR_PATHLEN (2<<8)
#define REPR_NAN (3<<8)
#define REPR_WORD (4<<8)

#define EXIST_EOF (1<<8)
#define EXIST_PREDICATE (2<<8)
#define EXIST_FUNCTION (3<<8)
#define EXIST_RESOURCE (4<<8)

#define PERM_READ (1<<8)
#define PERM_WRITE (2<<8)

/* SP_syserror_*() raises a system_error/1 with error information from
   the appropriate api, e.g. 'errno' in addition to the args. Must
   convert to string locally to dll. */
#if __WATCOMC__ || __BORLANDC__ || _MSC_VER || __EMX__ || MACINTOSH || __IBMC__ || __IBMCPP__
#define SP_syserror_clib(PRED,CALL) sp_syserror(API_CLIB, strerror(errno), PRED, CALL)
#else
#define SP_syserror_clib(PRED,CALL) sp_syserror(API_CLIB, NULL, PRED, CALL)
#endif
#define SP_syserror_win32(PRED,CALL) sp_syserror(API_WIN32, NULL, PRED, CALL)

#define API_CLIB 0
#define API_WIN32 1

/*----------------------------------------------------------------------*/
/* Undocumented. The following definitions should be avoided if possible. */

     /* Misc. prolog actions */

#define	SP_ACTION_TRACE		1 /* Switch to trace mode */
#define SP_ACTION_DEBUG		2 /* Switch to debug mode */
#define SP_ACTION_HALT		5 /* Halt Prolog execution */
#define SP_ACTION_ABORT		6 /* Generate a Prolog abort */

     /* Inquire prolog */

#define SP_INQUIRY_ARGC		1 /* Return main() argc */
#define SP_INQUIRY_ARGV		2 /* Return main() argv */
#define SP_INQUIRY_SYMBOLFILE	3 /* Not implemented */


#ifdef __cplusplus
};
#endif

#endif /* _SICSTUS_H */
